'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

exports.validateField = validateField;
exports.strlen = strlen;
exports.randomKey = randomKey;
exports.getDisplayName = getDisplayName;
exports.findAllReactComponentsByType = findAllReactComponentsByType;
exports.oneLengthChildrenToObject = oneLengthChildrenToObject;
exports.swapArrayItem = swapArrayItem;
exports.toUpperCaseByPosition = toUpperCaseByPosition;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _asyncValidator = require('async-validator');

var _asyncValidator2 = _interopRequireDefault(_asyncValidator);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 验证单个form组件值是否合法
 * @param  {string} id 即表单组件的name值
 * @param  {string} value 需验证的表单组件值
 * @param  {array}  rules async-validator rules
 * @return {function} 验证函数
 */
function validateField(id, value) {
  var rules = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  //返回promise有问题，很奇怪，this.setState会导致中文输入不行
  return function (successCallback, errorCallback) {
    if (!rules[0]) {
      successCallback && successCallback({});
      return;
    }
    var descriptor = {};
    descriptor[id] = rules;
    var validator = new _asyncValidator2.default(descriptor);
    validator.validate((0, _defineProperty3.default)({}, id, value), function (errors, fields) {
      if (errors) {
        errorCallback && errorCallback(errors);
      } else {
        successCallback && successCallback({});
      }
    });
  };
}
/**
 * 中英文字符长度，中文算两个长度
 * @param  {string} str 目标字符串
 * @return {number} 传入的字符串长度
 */
function strlen(str) {
  var len = 0;
  for (var i = 0; i < str.length; i++) {
    var c = str.charCodeAt(i);
    //单字节加1
    if (c >= 0x0001 && c <= 0x007e || 0xff60 <= c && c <= 0xff9f) {
      len++;
    } else {
      len += 2;
    }
  }
  return len;
}
function randomKey() {
  return Math.random().toString(36).substring(7).split('').join('');
}

/**
 * 获取组件displayName
 * @param  {object} Comp 指定的组件
 * @return {string}      组件displayName
 */
function getDisplayName(Comp) {
  if (!Comp) {
    return '';
  }
  if ((0, _isString2.default)('string')) {
    return Comp;
  }
  return Comp.displayName || 'Component';
}
/**
 * 根据组件displayName，递归查找React组件
 * @param  {array} children 必须是数组，jsx组件(this.props.children)
 * @param  {object} type 类型
 * @return {array} 返回匹配到的react组件，结构为
 *  {
 *    child: xxx,//当前组件
 *    index: xxx,//当前children中的位置，每个都可以能不一样
 *    children: xxx,//当前children，需要替换组件需要跟index配合。
 *  }
 */
function findAllReactComponentsByType(children, type) {
  if (!children[0]) {
    console.error('findAllReactComponentsByType第一个参数children必须是数组');
  }
  var result = [];
  var types = [];
  if ((0, _isArray2.default)(type)) {
    types = type.map(function (t) {
      return getDisplayName(t);
    });
  } else {
    types = [getDisplayName(type)];
  }
  var index = 0;
  _react2.default.Children.forEach(children, function (child) {
    var childType = child && child.type && child.type.displayName;
    if (types.indexOf(childType) !== -1) {
      //结构设置为这样，index和children可以配合替换当前匹配到的组件
      result.push({
        child: child,
        index: index,
        children: children
      });
    } else if (children[0]) {
      if (child && child.props && child.props.children) {
        var sortChildren = _react2.default.Children.toArray(child.props.children);
        children[index] = _react2.default.cloneElement(child, child.props, sortChildren);
        var result2 = findAllReactComponentsByType(children[index].props.children, type);
        result2.forEach(function (v) {
          result.push(v);
        });
      }
    }
    index++;
  });
  return result;
}
/**
 *
 *  转换一个数组的children为object对象
 *  配合findAllReactComponentsByType使用，children是经过React.Children.toArray转换后的
 */
function oneLengthChildrenToObject(children) {
  children.forEach(function (v, k) {
    if (v && v.props && v.props.children) {
      if (v.props.children.length === 1) {
        children[k] = _react2.default.cloneElement(v, v.props, v.props.children[0]);
        if (v.props.children.props && v.props.children.props.children) {
          oneLengthChildrenToObject(v.props.children.props.children);
        }
      }
    }
  });
}

/**
 * 交换数组指定的的两个键值位置,repalceIndex > index 下移，相反上移
 * @param {Array} arr 需要处理的数组
 * @param {index0} index 目标索引位置
 * @param {index1} repalceIndex 替换索引位置
 * @return {Array} 返回处理的数组
 */
function swapArrayItem(arr, index, repalceIndex) {
  arr[repalceIndex] = arr.splice(index, 1, arr[repalceIndex])[0];
  return arr;
}
/**
 * toUpperCase 小写转大写
 * @param  {string} string 传进来的字符串
 * @param  {int}  start  开始位置，默认0
 * @param  {Int}  end  介绍位置，默认1
 * @return {string}
 */
function toUpperCaseByPosition(string) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

  var str1 = string.substr(start, end).toUpperCase();
  var str2 = string.substr(end);
  return str1 + str2;
}